@@ -14,7 +14,7 @@ import logging
 import os
 import platform
 import socket
-# import retrying
+import uuid
 from waveletai import envs
 from waveletai.model import Model
 from contextlib import closing
@@ -29,7 +29,7 @@ from waveletai.model import ModelVersion
 from waveletai.app import App
 from waveletai.exceptions import LoginFailed, NotADirectory, MissingApiTokenException, FileUploadException, \
     InvalidAPIToken
-from waveletai.constants import DataType, ModelRegisterMode, Visibility, FeatureType, DataSource
+from waveletai.constants import DataType, ModelRegisterMode, Visibility, FeatureType, DataSource, DatasetImageType
 from waveletai.utils.storage_utils import UploadEntry, scan_upload_entries, SilentProgressIndicator, \
     LoggingProgressIndicator, upload_to_storage
 from waveletai.utils.datastream import FileStream, FileChunkStream, FileChunk
@@ -122,7 +122,13 @@ class HostedBackend(Backend):
         return data
 
     @with_api_exceptions_handler
-    def __init__(self, api_token=None):
+    def __init__(self, api_token=None, internal_login=True):
+        """
+        :param api_token:
+        :param internal_login: bool
+            `True`:内部登录，不记录登录日志；
+            `False`:外部登录，记录登录日志；
+        """
         if not api_token:
             api_token = os.getenv(envs.API_TOKEN)
         if api_token:
@@ -153,7 +159,7 @@ class HostedBackend(Backend):
             raise InvalidAPIToken()
         res = self._session.post(
             url=f'{self.api_address}/account/users/sdk/login/',
-            json={'api_token': api_token},
+            json={'api_token': api_token, 'internal_login': internal_login},
             headers=headers,
         )
         if res.json()["message"]:
@@ -200,7 +206,7 @@ class HostedBackend(Backend):
 
     @with_api_exceptions_handler
     @require_app
-    def update_dataset(self, dataset_id, name, visibility, desc):
+    def update_dataset(self, dataset_id, name, visibility, desc, label_type=None):
         """
         更新数据集信息
         :param desc: 数据集描述
@@ -216,6 +222,12 @@ class HostedBackend(Backend):
             jsondata['visibility'] = visibility
         if desc:
             jsondata['desc'] = desc
+        if label_type is not None:
+            if label_type != DatasetImageType.CLASSIFY.value\
+            and label_type != DatasetImageType.POLYGON.value\
+            and label_type != DatasetImageType.RECTANGLE.value:
+                raise Exception(f'图片数据集类型{label_type}不存在')
+            jsondata['json_data'] = {'labelType': label_type}
         res = self._session.put(url=f'{self.api_address}/{self.app_id}/data/dataset/{dataset_id}', json=jsondata)
         if res.json()['message']:
             raise Exception(res.json()['message'])
@@ -239,10 +251,17 @@ class HostedBackend(Backend):
     @with_api_exceptions_handler
     @require_app
     def create_dataset(self, name, path, data_type=DataType.TYPE_FILE.value, visibility=Visibility.PRIVATE.value,
-                       desc=None):
+                       desc=None, label_type=None):
+        json={'name': name, 'type': data_type, 'visibility': visibility, 'desc': desc}
+        if data_type == DataType.TYPE_IMAGE.value:
+            if label_type != DatasetImageType.CLASSIFY.value\
+            and label_type != DatasetImageType.POLYGON.value\
+            and label_type != DatasetImageType.RECTANGLE.value:
+                raise Exception(f'图片数据集类型{label_type}不存在')
+            json={'name': name, 'type': data_type, 'visibility': visibility, 'desc': desc, 'json_data':{'labelType': label_type}}
         res = self._session.post(
             url=f'{self.api_address}/{self.app_id}/data/dataset/',
-            json={'name': name, 'type': data_type, 'visibility': visibility, 'desc': desc},
+            json=json,
         )
         if res.json()["message"]:
             raise Exception(res.json()["message"])
@@ -287,16 +306,17 @@ class HostedBackend(Backend):
     # @retrying(stop_max_attempt_number=3)
     @require_app
     def _dataset_chunk_upload_loop(self, entry: UploadEntry, oid):
+        uid = uuid.uuid4()
         dataset_id = oid
         stream = FileChunkStream(entry)
         progress_indicator = LoggingProgressIndicator(stream.length, stream.filename)
         for ind, fc in enumerate(stream.generate(chunk_size=CHUNK_SIZE * 1024 * 1)):
             self._dataset_chunk_upload(fc, entry.filename, str(ind), entry.prefix.replace('\\', '/'),
-                                       progress_indicator, dataset_id)
+                                       progress_indicator, dataset_id, uid)
 
         res = self._session.get(url=f'{self.api_address}/{self.app_id}/data/dataset/upload/success',
                                 params={'filename': entry.filename, 'dataset_id': dataset_id,
-                                        'path': entry.prefix.replace('\\', '/'), 'final': 'true'})
+                                        'path': entry.prefix.replace('\\', '/'), 'uuid': uid, 'final': 'true'})
         if res.json()["message"]:
             _logger.error(f"file {entry.source_path} upload failed,{res.json()['message']}")
             raise FileUploadException(entry.filename)
@@ -306,9 +326,9 @@ class HostedBackend(Backend):
 
     @with_api_exceptions_handler
     @require_app
-    def _dataset_chunk_upload(self, file: FileChunk, filename, chunk, path, progress_indicator, oid):
+    def _dataset_chunk_upload(self, file: FileChunk, filename, chunk, path, progress_indicator, oid, uid):
         dataset_id = oid
-        payload = {"chunk": chunk, "path": path, "dataset_id": dataset_id}
+        payload = {"chunk": chunk, "path": path, "dataset_id": dataset_id, "uuid": uid}
         res = self._session.post(url=f'{self.api_address}/{self.app_id}/data/dataset/upload',
                                  files=[('file', (file.data))],
                                  data=payload)
@@ -338,14 +358,15 @@ class HostedBackend(Backend):
     @with_api_exceptions_handler
     @require_model
     def _model_chunk_upload_loop(self, entry: UploadEntry, oid):
+        uid = uuid.uuid4()
         model_repo_id = oid
         stream = FileChunkStream(entry)
         progress_indicator = LoggingProgressIndicator(stream.length, stream.filename)
         for ind, fc in enumerate(stream.generate(chunk_size=CHUNK_SIZE * 1024 * 20)):
-            self._model_chunk_upload(fc, entry.filename, str(ind), entry.prefix, progress_indicator, model_repo_id)
+            self._model_chunk_upload(fc, entry.filename, str(ind), entry.prefix, progress_indicator, model_repo_id, uid)
 
         res = self._session.get(url=f'{self.api_address}/{self.model_id}/deploy/model/upload/success',
-                                params={'filename': entry.filename, 'task_id': model_repo_id, 'path': entry.prefix})
+                                params={'filename': entry.filename, 'uuid': uid, 'task_id': model_repo_id, 'path': entry.prefix})
         if res.json()["message"]:
             _logger.error(f"file {entry.source_path} upload failed,{res.json()['message']}")
             raise FileUploadException(entry.filename)
@@ -355,9 +376,9 @@ class HostedBackend(Backend):
 
     # @retrying(stop_max_attempt_number=3)
     @require_model
-    def _model_chunk_upload(self, file: FileChunk, filename, chunk, path, progress_indicator, oid):
+    def _model_chunk_upload(self, file: FileChunk, filename, chunk, path, progress_indicator, oid, uid):
         model_repo_id = oid
-        payload = {"chunk": chunk, "path": path, "task_id": model_repo_id}
+        payload = {"chunk": chunk, "path": path, "uuid": uid, "task_id": model_repo_id}
         res = self._session.post(url=f'{self.api_address}/{self.model_id}/deploy/model/upload',
                                  files=[('file', (file.data))],
                                  data=payload)
@@ -521,8 +542,24 @@ class HostedBackend(Backend):
         except Exception as e:
             raise e
 
+    @with_api_exceptions_handler
+    def _check_ip(self, ipAddr):
+        import re
+        compile_ip = re.compile(
+            '^(1\d{2}|2[0-4]\d|25[0-5]|[1-9]\d|[1-9])\.(1\d{2}|2[0-4]\d|25[0-5]|[1-9]\d|\d)\.(1\d{2}|2[0-4]\d|25[0-5]|[1-9]\d|\d)\.(1\d{2}|2[0-4]\d|25[0-5]|[1-9]\d|\d)$')
+        if compile_ip.match(ipAddr):
+            return True
+        else:
+            return False
+
     @with_api_exceptions_handler
     def _get_ip(self):
+        from urllib.parse import urlparse
+        hostname = urlparse(self.api_address).hostname
+        if self._check_ip(hostname):
+            import ipaddress
+            if ipaddress.ip_address(hostname).is_private:
+                return hostname
         try:
             s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
             s.connect(('8.8.8.8', 80))
@@ -567,9 +604,7 @@ class HostedBackend(Backend):
         :param dataset_id:
         :return: 返回文件，图片数据集文件列表
         """
-        client_ip = self._get_ip()
-        res = self._session.get(url=f'{self.api_address}/{self.app_id}/data/dataset/{dataset_id}/asset?page=-1',
-                                headers={"client-ip": client_ip})
+        res = self._session.get(url=f'{self.api_address}/{self.app_id}/data/dataset/{dataset_id}/asset?page=-1')
         if res.json()["message"]:
             raise Exception(res.json()["message"])
         artifacts = []
@@ -894,7 +929,11 @@ class HostedBackend(Backend):
             minio_path = minio_path[0] + '/resource'
             s3path = self._get_repo_s3path(repo_id)
             if s3path is None:
-                return 's3path不存在'
+                return '当前模型库下无文件，请检查模型库ID或模型库文件是否已经存在'
+            s3path = s3path.split("/")
+            s3path.pop()
+            s3path.pop(0)
+            model_path = "/".join(s3path)
             with Pool(5) as pool:
                 resultinfo, dir = self._list_dir_model_artifacts(repo_id, None, [], [])
                 for i in dir:
@@ -906,7 +945,7 @@ class HostedBackend(Backend):
                     result_path = result.rsplit('/', 1)
                     des_path = result_path[0]
                     result_path = result_path[-1]
-                    file_path = minio_path + s3path + '/' + result_path
+                    file_path = minio_path + model_path + '/' + des_path + "/" + result_path
                     urls.append((file_path, destination + '/' + des_path, result_path))
                 pool.map(self._url_download, urls)
                 pool.close()
@@ -967,12 +1006,11 @@ class HostedBackend(Backend):
     @with_api_exceptions_handler
     @require_app
     def _get_feature_zip_path(self, feature_id, feature_type):
-        client_ip = self._get_ip()
         if feature_type is None:
             feature_type = FeatureType.FILE.value
         for i in range(100):
             res = self._session.get(
-                url=f'{self.api_address}/{self.app_id}/data/feature/{feature_id}/asset?type={feature_type}', headers={"client-ip": client_ip})
+                url=f'{self.api_address}/{self.app_id}/data/feature/{feature_id}/asset?type={feature_type}')
             if res.json()["message"]:
                 raise Exception(res.json()["message"])
             data = res.json()['data']
@@ -1457,12 +1495,11 @@ class HostedBackend(Backend):
             os.makedirs(destination_dir)
         elif not os.path.isdir(destination_dir):
             raise NotADirectory(destination_dir)
-        client_ip = self._get_ip()
         try:
             if DataSource.MAMMUT.value == datasource or DataSource.WTHINGS.value == datasource:
                 res = self._session.post(
                     url=f'{self.api_address}/{self.app_id}/data/dataset/netease/data/sdk/{dataset_id}',
-                    json={"query": query, "datasource": datasource}, headers={"client-ip": client_ip})
+                    json={"query": query, "datasource": datasource})
                 if res.json()["message"]:
                     return res.json()["message"]
                 path = res.json()['data']
